@@ -1,9 +1,14 @@
-from fps.config import PluginModel  # type: ignore
+from fps.config import PluginModel, get_config  # type: ignore
 from fps.hooks import register_config, register_plugin_name  # type: ignore
 
 
 class JupyterLabConfig(PluginModel):
     collaborative: bool = False
+    base_url: str = "/"
+
+
+def get_jlab_config():
+    return get_config(JupyterLabConfig)
 
 
 c = register_config(JupyterLabConfig)
