@@ -1,23 +1,32 @@
 import json
-import pathlib
+from pathlib import Path
 import sys
 from glob import glob
 from http import HTTPStatus
-from typing import Dict, Any
+from typing import Optional
 
+from pydantic import UUID4
 import jupyterlab  # type: ignore
-from fastapi import APIRouter, Response
-from fastapi.responses import HTMLResponse, RedirectResponse
+import jupyverse  # type: ignore
+from fastapi import APIRouter, Response, Depends, status
+from fastapi.responses import HTMLResponse, FileResponse, RedirectResponse
 from fastapi.staticfiles import StaticFiles
 from starlette.requests import Request  # type: ignore
-from fps.config import Config  # type: ignore
 from fps.hooks import register_router  # type: ignore
 
-from .config import JupyterLabConfig
+from fps_auth.routes import (  # type: ignore
+    current_user,
+    user_db,
+    cookie_authentication,
+    LoginCookieAuthentication,
+)
+from fps_auth.models import User  # type: ignore
+from fps_auth.config import get_auth_config  # type: ignore
+
+from .config import get_jlab_config
 
 router = APIRouter()
-prefix_dir: pathlib.Path = pathlib.Path(sys.prefix)
-config = Config(JupyterLabConfig)
+prefix_dir: Path = Path(sys.prefix)
 
 router.mount(
     "/static/lab",
@@ -55,47 +64,51 @@ for path in glob(
         name=name,
     )
 
-WORKSPACE = {
-    "data": {
-        "layout-restorer:data": {
-            "main": {
-                "dock": {"type": "tab-area", "currentIndex": 0, "widgets": []},
-                "current": "",
-            },
-            "down": {"size": 0, "widgets": []},
-            "left": {
-                "collapsed": False,
-                "current": "filebrowser",
-                "widgets": [
-                    "filebrowser",
-                    "running-sessions",
-                    "@jupyterlab/toc:plugin",
-                    "extensionmanager.main-view",
-                ],
-            },
-            "right": {"collapsed": True, "widgets": []},
-            "relativeSizes": [0.22859744990892533, 0.7714025500910747, 0],
-        },
-        "@jupyterlab/docprovider:yprovider:user": "undefined,#4BA749",
-    },
-    "metadata": {
-        "id": "default",
-        "last_modified": "2021-08-10T10:47:58.741703+00:00",
-        "created": "2021-08-10T10:47:58.741703+00:00",
-    },
-}
-
-SETTINGS: Dict[str, Any] = {}
-
 
 @router.get("/")
-async def get_root():
-    return RedirectResponse("/lab")
+async def get_root(
+    response: Response,
+    token: Optional[UUID4] = None,
+    auth_config=Depends(get_auth_config),
+    jlab_config=Depends(get_jlab_config),
+):
+    if token and auth_config.mode == "token":
+        user = await user_db.get(token)
+        if user:
+            await super(
+                LoginCookieAuthentication, cookie_authentication
+            ).get_login_response(user, response)
+    # auto redirect
+    response.status_code = status.HTTP_302_FOUND
+    response.headers["Location"] = jlab_config.base_url + "lab"
 
 
 @router.get("/lab")
-async def get_lab():
-    return HTMLResponse(get_index("default", config.collaborative))
+async def get_lab(
+    user: User = Depends(current_user()), jlab_config=Depends(get_jlab_config)
+):
+    return HTMLResponse(
+        get_index("default", jlab_config.collaborative, jlab_config.base_url)
+    )
+
+
+@router.get("/lab/tree/{path:path}")
+async def load_workspace(path, jlab_config=Depends(get_jlab_config)):
+    return HTMLResponse(
+        get_index("default", jlab_config.collaborative, jlab_config.base_url)
+    )
+
+
+@router.get("/favicon.ico")
+async def get_favicon():
+    return FileResponse(Path(jupyverse.__file__).parent / "static" / "favicon.ico")
+
+
+@router.get("/static/notebook/components/MathJax/{rest_of_path:path}")
+async def get_mathjax(rest_of_path):
+    return RedirectResponse(
+        "https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.7/" + rest_of_path
+    )
 
 
 @router.get("/lab/api/listings/@jupyterlab/extensionmanager-extension/listings.json")
@@ -109,22 +122,30 @@ async def get_listings():
 
 
 @router.get("/lab/api/workspaces/{name}")
-async def get_workspace_data(name):
-    return WORKSPACE
+async def get_workspace_data(user: User = Depends(current_user(optional=True))):
+    if user:
+        return json.loads(user.workspace)
+    return {}
 
 
 @router.put(
     "/lab/api/workspaces/{name}",
     status_code=204,
 )
-async def set_workspace(request: Request):
-    WORKSPACE.update(await request.json())
+async def set_workspace(
+    request: Request,
+    user: User = Depends(current_user()),
+):
+    user.workspace = await request.body()
+    await user_db.update(user)
     return Response(status_code=HTTPStatus.NO_CONTENT.value)
 
 
 @router.get("/lab/workspaces/{name}", response_class=HTMLResponse)
-async def get_workspace(name):
-    return get_index(name, config.collaborative)
+async def get_workspace(
+    name, user: User = Depends(current_user()), jlab_config=Depends(get_jlab_config)
+):
+    return get_index(name, jlab_config.collaborative, jlab_config.base_url)
 
 
 @router.get("/lab/api/translations")
@@ -133,29 +154,41 @@ async def get_translations():
 
 
 @router.get("/lab/api/translations/{language}")
-async def get_translation(language):
+async def get_translation(
+    language,
+):
     return {}
 
 
-@router.get("/lab/api/settings/@jupyterlab/{name0}:{name1}")
-async def get_setting(name0, name1):
+@router.get("/lab/api/settings/{name0}/{name1}:{name2}")
+async def get_setting(
+    name0,
+    name1,
+    name2,
+    user: User = Depends(current_user(optional=True)),
+):
     with open(
         prefix_dir / "share" / "jupyter" / "lab" / "static" / "package.json"
     ) as f:
         package = json.load(f)
+    if name0 == "@jupyterlab":
+        lab_or_extensions = Path("lab")
+    else:
+        lab_or_extensions = Path("labextensions") / name0 / name1
     with open(
         prefix_dir
         / "share"
         / "jupyter"
-        / "lab"
+        / lab_or_extensions
         / "schemas"
-        / "@jupyterlab"
         / name0
-        / f"{name1}.json"
+        / name1
+        / f"{name2}.json"
     ) as f:
         schema = json.load(f)
+    key = f"{name1}:{name2}"
     result = {
-        "id": f"@jupyterlab/{name0}:{name1}",
+        "id": f"@jupyterlab/{key}",
         "schema": schema,
         "version": package["version"],
         "raw": "{}",
@@ -163,8 +196,10 @@ async def get_setting(name0, name1):
         "last_modified": None,
         "created": None,
     }
-    if f"{name0}:{name1}" in SETTINGS:
-        result.update(SETTINGS[f"{name0}:{name1}"])
+    if user:
+        settings = json.loads(user.settings)
+        if key in settings:
+            result.update(settings[key])
     return result
 
 
@@ -172,29 +207,48 @@ async def get_setting(name0, name1):
     "/lab/api/settings/@jupyterlab/{name0}:{name1}",
     status_code=204,
 )
-async def change_setting(request: Request, name0, name1):
-    SETTINGS[f"{name0}:{name1}"] = await request.json()
+async def change_setting(
+    request: Request,
+    name0,
+    name1,
+    user: User = Depends(current_user()),
+):
+    settings = json.loads(user.settings)
+    settings[f"{name0}:{name1}"] = await request.json()
+    user.settings = json.dumps(settings)
+    await user_db.update(user)
     return Response(status_code=HTTPStatus.NO_CONTENT.value)
 
 
 @router.get("/lab/api/settings")
-async def get_settings():
+async def get_settings(user: User = Depends(current_user(optional=True))):
+    with open(
+        prefix_dir / "share" / "jupyter" / "lab" / "static" / "package.json"
+    ) as f:
+        package = json.load(f)
+    if user:
+        user_settings = json.loads(user.settings)
+    else:
+        user_settings = {}
     settings = []
     for path in (
         prefix_dir / "share" / "jupyter" / "lab" / "schemas" / "@jupyterlab"
     ).glob("*/*.json"):
         with open(path) as f:
             schema = json.load(f)
+        key = f"{path.parent.name}:{path.stem}"
         setting = {
-            "id": f"@jupyterlab/{path.parent.name}:{path.stem}",
+            "id": f"@jupyterlab/{key}",
             "schema": schema,
-            "version": "3.1.0-rc.1",
+            "version": package["version"],
             "raw": "{}",
             "settings": {},
             "warning": None,
             "last_modified": None,
             "created": None,
         }
+        if key in user_settings:
+            setting.update(user_settings[key])
         settings.append(setting)
     return {"settings": settings}
 
@@ -232,12 +286,11 @@ INDEX_HTML = """\
 """
 
 
-def get_index(workspace, collaborative):
+def get_index(workspace, collaborative, base_url="/"):
     for path in (prefix_dir / "share" / "jupyter" / "lab" / "static").glob("main.*.js"):
         main_id = path.name.split(".")[1]
         break
-    base_url = "/"
-    full_static_url = "/static/lab"
+    full_static_url = f"{base_url}static/lab"
     page_config = {
         "appName": "JupyterLab",
         "appNamespace": "lab",
@@ -285,7 +338,6 @@ def get_index(workspace, collaborative):
     }
     index = (
         INDEX_HTML.replace("PAGE_CONFIG", json.dumps(page_config))
-        .replace("BASE_URL", base_url)
         .replace("FULL_STATIC_URL", full_static_url)
         .replace("MAIN_ID", main_id)
     )
